// ============================================================================
// Project:	Deductive Database
// Filename:	rel_ii.cpp
// Purpose:	Relation with two integer columns, no index.
// Last Change:	11.09.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2010-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "str.h"
#ifdef	VER_DUMP
#include "err.h"
#endif
#include "check.h"
#include "flexarr.h"

#include "rel_ii.h"

#include <iostream>

//=============================================================================
// Constructor and Destructor:
//=============================================================================


//-----------------------------------------------------------------------------
// Constructor:
//-----------------------------------------------------------------------------

rel_ii_c::rel_ii_c(str_t rel_name) :
		MemPool(rel_name), TupleArr(&MemPool)
{
	// Set Magic number:
	CHECK_CODE(Magic = REL_II_MAGIC);

	// Check parameter:
	CHECK(rel_name != STR_NULL,
			"rel_ii_c::constructor: rel_name is NULL");

	// Initialize attributes (which are not nested objects):
	Name    = rel_name;
	NumRows = 0;
}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

rel_ii_c::~rel_ii_c()
{
	// Check this object:
	CHECK_VALID("rel_ii_c::destructor");

	// Make this object invalid:
	CHECK_CODE(Magic = 0);
}

//-----------------------------------------------------------------------------
// insert: Insert tuple into relation.
//-----------------------------------------------------------------------------

bool rel_ii_c::insert(int c1, int c2)
{
	// Check this object:
	CHECK_VALID("rel_ii_c::insert");

	// Simply add columns to flexible array, first c1, then c2:
	if(TupleArr.append(c1) < 0)
		return false;
	if(TupleArr.append(c2) < 0) {
		// TODO: Should remove last element, but cannot.
		// TODO: Solution: Use flexarray of int pair objects,
		// 		not ints.
		return false;
	}

	// Ok:
	NumRows++;
	return true;
}

//=============================================================================
// Debugging Support:
//=============================================================================

//-----------------------------------------------------------------------------
// check: Check the integrity of this object, return error message or NULL.
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
str_t	rel_ii_c::check() const
{
	// Check magic number:
	if(Magic != REL_II_MAGIC)
		return "wrong magic number";

	// Check attributes:
	if(Name == STR_NULL)
		return "relation name is null";
	if(NumRows < 0)
		return "number of rows is negative";

	// Check links from index array to value list array:
	if(TupleArr.length() != 2 * NumRows) {
		dump();
		return "length of tuple array is inconsistent with NumRows";
	}

	// Check sub-objects:
	CHECK_SUBOBJ(MemPool, "rel_ii_c::check");
	CHECK_SUBOBJ(TupleArr, "rel_ii_c::check");

	// Ok:
	return 0;
}
#endif


//-----------------------------------------------------------------------------
// dump: Show data structure.
//-----------------------------------------------------------------------------

#ifdef	VER_DUMP
void	rel_ii_c::dump(str_t headline) const
{
	// Headline:
	if(headline == STR_NULL)
		headline = "Relation (with two integer columns, no index)";
	err_c::dump_begin(headline);

	// Basic information:
	err_c::dump_str("Name:           '");
	err_c::dump_str(Name);
	err_c::dump_str("'");
	err_c::dump_nl();
	err_c::dump_str("NumRows:         ");
	err_c::dump_int(NumRows);
	err_c::dump_nl();
	err_c::dump_str("Memory:          ");
	int pages = MemPool.num_alloc_pages();
	err_c::dump_int(pages);
	err_c::dump_str(" Pages [");
	err_c::dump_int(PAGE_KB(pages));
	err_c::dump_str(" KB]");
	err_c::dump_nl();
	err_c::dump_str("TupleArr Length: ");
	err_c::dump_int(TupleArr.length());
	err_c::dump_nl();
	err_c::dump_str("Object Size:     ");
	err_c::dump_uint(static_cast<unsigned int>(sizeof(rel_ii_c)));
	err_c::dump_str(" Bytes");
	err_c::dump_nl();
	err_c::dump_nl();

	// Show entire tuple array:
	err_c::dump_str("Tuple Array:");
	err_c::dump_nl();
	for(int i = 0; i < TupleArr.length(); i++) {
		int n = TupleArr.get(i);
		if(i % 2 == 0) {
			err_c::dump_str("  ");
			err_c::dump_str(str_fill(i, 5));
			err_c::dump_int(i);
			err_c::dump_str(": Col_1 = ");
			err_c::dump_str(str_fill(n, 6));
			err_c::dump_int(n);
			err_c::dump_str("  ");
		}
		else {
			err_c::dump_str(", Col_2 = ");
			err_c::dump_str(str_fill(n, 6));
			err_c::dump_int(n);
			err_c::dump_nl();
		}
	}
	err_c::dump_nl();


	// Done:
	err_c::dump_end();
}
#endif

